'************************************************************************************* 
' 
' Peter Mather
' 
' 2.0 
' 2015-20-6 
' 
' Graphics library with drivers for SSD1983 display 
' 9 bit parallel output from Micromite MX170 - flexible pinout
'
' drawing primitives assume a graphics display where top left is point 0, 0
'
' set screen orientation: sub SSD.setrot(x) portrait, portrait180, landscape, landscape180
' plot pixel: sub SSD_pixel(x, y, colour) x, y coordinates of pixel, colour
' draw circle: sub SSD.dcirc(x0, y0, r, colour) x, y coordinates of centre, radius, colour
' draw filled circle: sub SSD.fcirc(x0, y0, r, colour) x, y coordinates of centre, radius, colour
' draw triangle:  sub SSD.dtri(x0, y0, x1, y1, 	x2, y2, colour) three coordinate pairs, colour
' draw filled triangle sub SSD.ftri (x0, y0, x1, y1, x2, y2, colour) three coordinate pairs, colour
' print string: sub SSD.pstring(font, xx0, yy0, text$, sz, orientation, forecolour, backcolour) font number, x, y coordinates of top left of first character, text string, size, character orientation,  foreground colour,  background colour
' print bitmap: sub SSD.pbitmap(xx0, yy0, width, height, scale, forecolour, backcolour) x, y coordinates of top left of first character, bitmap, size, character orientation,  foreground colour,  background colour
' draw line: sub SSD.dline(xx0, yy0, xx1, yy1, colour) x, y of start line, x, y of end line, colour
' draw rectangle: sub SSD.drect(x, y, w, h, colour) x, y coordinate of top left, width, height, colour
' draw filled rectangle: sub SSD.frect(x, y, w, h, colour) x, y coordinate of top left, width, height, colour
' draw rounded rectangle: sub SSD.drndrect(x, y, w, h, r, colour) x, y coordinates of top left, width, height, radius of corner, colour
' draw filled rounded rectangle: sub SSD.frndrect(x, y, w, h, r, colour) x, y coordinates of top left, width, height, radius of corner, colour
' draw bitmap: sub SSD.bitmap(x, y , w , h , size, forecolour, backcolour, bitmap()) x, y coordinates of top left, width, height, scale, foreground colour, background colour, bitmap as integer array
'
option explicit
option default NONE
const displaysize=4.3 'choose the size of display 4.3 or 5 or 7
'
' Define the pins to use for all connections to the display
'
Const RS=12 ' define the pin to use for RS
const WR=5  ' etc
const DB0=25
const DB1=26
const DB2=27
CONST DB3=36
CONST DB4=37
CONST DB5=38
CONST DB6=2
CONST DB7=3
CONST DB8=4
const Master_reset=32 'define the pin to use for REST
' Connect RD to VCC
' Connect CS to GND
'

cpu 48
  print "start Init"
  SSD.initdisplay
  print "Start main"

Do
   SSD_test
loop

end
'
' 
' Display specific routines
'
sub SSD.initdisplay
'
  dim integer SSD.pix=0
  dim integer SSD.ffr=1
  dim integer SSD.filltri=2
  dim integer SSD.drawline=3
  dim integer SSD.writecommand=4
  dim integer SSD.writedata=5
  dim integer SSD.fpstring=7
  dim integer SSD.fcirch=8
  dim integer SSD.dcirch=9
  dim integer SSD.Bits=10
  dim integer landscape=2
  dim integer landscape180=3
  dim integer portrait=0
  dim integer portrait180=1
  dim integer displaywidth,displayheight
  dim integer  SSD1963_PAG_COL_ADDR_ORDER = (1 << 5)
  dim integer  SSD1963_COL_ADDR_ORDER     = (1 << 6)
  dim integer  SSD1963_PAGE_ADDR_ORDER    = (1 << 7)
  ' Global variables
  dim integer SSD_params(17) ' Global variables
  DIM integer SSD_wi=0,SSD_ht=1,SSD_rot=2,SSD_nr=3,SSD_X=4,SSD_Y=5,SSD_seed=6,SSD_RS=7
  dim integer SSD_WR=8, SSD_DB0=9, SSD_DB1=10, SSD_DB2=11, SSD_DB3=12, SSD_DB4=13, SSD_DB5=14, SSD_DB6=15, SSD_DB7=16, SSD_DB8=17
  local integer i,j
  pin(WR)=1: setpin WR,DOUT 'C9 - connect to WR
  setpin DB0,dout: pin(DB0)=0 'C0 - connect to D0
  setpin DB1,dout: pin(DB0)=0 'C1 - connect to D1
  setpin DB2,dout: pin(DB0)=0 'C2 - connect to D2
  setpin DB3,dout: pin(DB0)=0 'C3 - connect to D3
  setpin DB4,dout: pin(DB0)=0 'c4 - connect to D4
  setpin DB5,dout: pin(DB0)=0 'C5 - connect to D5
  setpin DB6,dout: pin(DB0)=0 'C6 - connect to D6
  setpin DB7,dout: pin(DB0)=0 'C7 - connect to D7
  setpin DB8,dout: pin(DB0)=0 'C8 - connect to D8
  pin(Master_reset)=1:  SETPIN Master_reset,dout
  pin(RS)=1: setpin RS, DOUT
  SSD_params(SSD_RS)=RS 'store pin alocation for use in Cfunction
  SSD_params(SSD_WR)=wr
  SSD_params(SSD_DB0)=DB0
  SSD_params(SSD_DB1)=DB1
  SSD_params(SSD_DB2)=DB2
  SSD_params(SSD_DB3)=DB3
  SSD_params(SSD_DB4)=DB4
  SSD_params(SSD_DB5)=DB5
  SSD_params(SSD_DB6)=DB6
  SSD_params(SSD_DB7)=DB7
  SSD_params(SSD_DB8)=DB8
  SSD_params(SSD_seed)=0 ' store current state of data pins
  if displaysize=4.3 then
    displaywidth=480
    displayheight=272
  else
    displaywidth=800
    displayheight=480
  endif
  SSD_params(SSD_wi)=displaywidth
  SSD_params(SSD_ht)=displayheight
  pin(Master_RESET)=0
  pause 100
  pin(Master_RESET)=1
  pause 100
	SSDwritecommand(&HE2)		'PLL multiplier, set PLL clock to 120M
	if displaysize=4.3 then
    SSDwritedata(&H23)	   
  else
    SSDwritedata(&H1E)	    
  endif
	SSDwritedata(&H02)
	SSDwritedata(&H54)
	SSDwritecommand(&HE0)		' PLL enable
	SSDwritedata(&H01)
	pause 10)
	SSDwritecommand(&HE0)
	SSDwritedata(&H03)
	pause 10)
	SSDwritecommand(&H01)		' software reset
	pause 100)
	SSDwritecommand(&HE6)		'PLL setting for PCLK, depends on resolution
	if displaysize=4.3 then
    SSDwritedata(&H01)
    SSDwritedata(&H1F)
  else
    SSDwritedata(&H03)
    SSDwritedata(&HFF)
  endif
	SSDwritedata(&HFF)

	SSDwritecommand(&HB0)		'LCD SPECIFICATION
	if displaysize=4.3 then
    SSDwritedata(&H20)
    SSDwritedata(&H00)
    SSDwritedata(&H01)	
    SSDwritedata(&HDF)
    SSDwritedata(&H01)
    SSDwritedata(&H0F)
    SSDwritedata(&H00)
    SSDwritecommand(&HB4)	
    SSDwritedata(&H02)
    SSDwritedata(&H13)
    SSDwritedata(&H00)
    SSDwritedata(&H08)
    SSDwritedata(&H2B)
    SSDwritedata(&H00)
    SSDwritedata(&H02)
    SSDwritedata(&H00)
    SSDwritecommand(&HB6)	
    SSDwritedata(&H01)
    SSDwritedata(&H20)
    SSDwritedata(&H00)
    SSDwritedata(&H04)
    SSDwritedata(&H0C)
    SSDwritedata(&H00)
    SSDwritedata(&H02)
  else
    SSDwritedata(&H24)
    SSDwritedata(&H00)
    SSDwritedata(&H03)
    SSDwritedata(&H1F)
    SSDwritedata(&H01)
    SSDwritedata(&HDF)
    SSDwritedata(&H00)
    SSDwritecommand(&HB4)
    SSDwritedata(&H03)
    SSDwritedata(&HA0)
    SSDwritedata(&H00)
    SSDwritedata(&H2E)
    SSDwritedata(&H30)
    SSDwritedata(&H00)
    SSDwritedata(&H0F)
    SSDwritedata(&H00)
    SSDwritecommand(&HB6)	
    SSDwritedata(&H02)
    SSDwritedata(&H0D)
    SSDwritedata(&H00)
    SSDwritedata(&H10)
    SSDwritedata(&H10)
    SSDwritedata(&H00)
    SSDwritedata(&H08)
  endif
	SSDwritecommand(&HBA)
	SSDwritedata(&H0F)		'GPIO[3:0] out 1
	SSDwritecommand(&HB8)
	SSDwritedata(&H07)	    'GPIO3=input, GPIO[2:0]=output
	SSDwritedata(&H01)		'GPIO0 normal
	SSDwritecommand(&H36)		'rotation
	SSDwritedata(&B00000011)
	SSDwritecommand(&HF0)		'pixel data interface
	SSDwritedata(&H06)
	SSDwritecommand(&H3A)		'pixel data interface
	SSDwritedata(&B01100000)
	SSDwritecommand(&H26)		'gamma curve
	SSDwritedata(&B00000001)
	pause 1
	SSDwritecommand(&H29)		'display on
	SSDwritecommand(&HBE)		'set PWM for B/L
	SSDwritedata(&H06)
	SSDwritedata(&Hf0)
	SSDwritedata(&H01)
	SSDwritedata(&Hf0)
	SSDwritedata(&H00)
	SSDwritedata(&H00)
	SSDwritecommand(&Hd0) 
	SSDwritedata(&H0d)	
	SSDwritecommand(&H2C) 
end sub
'
sub SSDwritedata(d as integer)
  local integer i=ssddraw(SSD.writedata, d, SSD_params())
end sub
'
sub SSDwritecommand(d as integer)
  local integer i=ssddraw(SSD.writecommand, d, SSD_params())
end sub
'
sub SSD.fillscreen(scol as integer)
  local integer i=SSD_params(SSD_rot)
  SSD_params(SSD_rot)=landscape
  local integer j=ssddraw(SSD.ffr,scol, 0,0,SSD_params(SSD_wi),SSD_params(SSD_ht), SSD_params())
  SSD_params(SSD_rot)=i
end sub
'
' Generic drawing routines
'
sub SSD.dcirc(x0 as integer, y0 as integer, r as integer, colour as integer) 'x, y coordinates of centre, radius
  local integer i=ssddraw(SSD.dcirch, colour,x0,y0,r,31,SSD_params())

end sub
 '
sub SSD.dtri(x0 as integer, y0 as integer, x1 as integer, y1 as integer, x2 as integer, y2 as integer, colour as integer) 'three coordinate pairs
  SSD.dline(x0, y0, x1, y1, colour)
  SSD.dline(x1, y1, x2, y2, colour)
  SSD.dline(x2, y2, x0, y0, colour)
end sub
'
sub SSD.pixel(x0 as integer, y0 as integer,  colour as integer) 'three coordinate pairs
  local integer i=ssddraw(SSD.pix, colour,x0,y0,SSD_params())
end sub
 '
sub SSD.ftri(x0 as integer, y0 as integer, x1 as integer, y1 as integer, x2 as integer, y2 as integer, colour as integer) 'three coordinate pairs
  local integer i=ssddraw(SSD.filltri, x0,y0,x1,y1,x2,y2,colour,SSD_params())
end sub
'
sub SSD.pstring(font as integer, xx0 as integer, yy0 as integer, text$, sz as integer, fontcolour as integer, backgroundcolour as integer) 'x, y coordinates of top left of first character, text string, size, orientation
  local integer j=font and 1
  local integer i=ssddraw(SSD.fpstring,j,xx0,yy0,sz,fontcolour,backgroundcolour,SSD_params(),text$)
end sub
'
sub SSD.dline(xx0 as integer, yy0 as integer, xx1 as integer, yy1 as integer, colour as integer) ' x, y of start line, x, y of end line
   local integer dp=ssddraw(SSD.drawline, colour, xx0, yy0, xx1, yy1,  SSD_params())
end sub
'
sub SSD.drect(x as integer, y as integer, w as integer, h as integer, rcolour as integer) 'x, y coordinate of top left, wi, ht
  local integer a, dp
  dp=ssddraw(SSD.ffr, rcolour, x, y, w, 1,  SSD_params()) 
  a=y+h-1
  dp=ssddraw(SSD.ffr, rcolour, x, a, w, 1,  SSD_params()) 
  dp=ssddraw(SSD.ffr, rcolour, x, y, 1, h,  SSD_params()) 
  a=x+w-1
  dp=ssddraw(SSD.ffr, rcolour, a, y, 1, h,  SSD_params()) 
end sub
'
sub SSD.bitmap(x as integer, y as integer, w as integer, h as integer, size as integer, fcolour as integer, bcolour as integer, array() as integer) 'x, y coordinate of top left, wi, ht
  local integer  j=ssddraw(SSD.Bits, x,y,w,h,size,fcolour,bcolour,array(),SSD_params())  
end sub
'
sub SSD.frect(x as integer, y as integer, w as integer, h as integer, rcolour as integer) 'x, y coordinate of top left, width, height
  local integer dp
  dp=ssddraw(SSD.ffr, rcolour, x, y, w, h,  SSD_params()) 
end sub
'
sub SSD.fcirc(x0 as integer, y0 as integer, r as integer, ccolour as integer)  'x, y coordinates of centre, radius
  local integer dp
  dp=ssddraw(SSD.ffr, ccolour, x0, y0-r, 1, 2*r+1,  SSD_params())
  dp=ssddraw(SSD.fcircH,x0, y0, r, 3, 0, ccolour, SSD_params())
end sub
 '
sub SSD.drndrect(x as integer, y as integer, w as integer, h as integer, r as integer, rcolour as integer) 'x, y coordinates of top left, width, height, radius of corner
  local integer dp
  dp=ssddraw(SSD.ffr, rcolour, x+r, y, w-2*r, 1,  SSD_params()) 
  dp=ssddraw(SSD.ffr, rcolour, x+r, y+h-1, w-2*r, 1,  SSD_params()) 
  dp=ssddraw(SSD.ffr, rcolour, x, y+r, 1, h-2*r,  SSD_params()) 
  dp=ssddraw(SSD.ffr, rcolour, x+w-1, y+r, 1, h-2*r,  SSD_params()) 
  dp=ssddraw(SSD.dcirch, rcolour, x+r, y+r, r, 1,SSD_params())
  dp=ssddraw(SSD.dcirch,  rcolour, x+w-r-1, y+r, r, 2,SSD_params())
  dp=ssddraw(SSD.dcirch,  rcolour, x+w-r-1, y+h-r-1, r, 4,SSD_params())
  dp=ssddraw(SSD.dcirch,  rcolour, x+r, y+h-r-1, r, 8,SSD_params())
end sub
'
sub SSD.frndrect(x as integer, y as integer, w as integer, h as integer, r as integer, colour as integer) 'x, y coordinates of top left, width, height, radius of corner
  local integer dp
  dp=ssddraw(SSD.ffr, colour, x+r, y, w-2*r, h,  SSD_params()) 
  dp=ssddraw(SSD.fcircH,x+w-r-1, y+r, r, 1, h-2*r-1, colour, SSD_params())
  dp=ssddraw(SSD.fcircH,x+r, y+r, r, 2, h-2*r-1, colour, SSD_params())
end sub
'
sub SSD.bitmap(x as integer, y as integer, w as integer, h as integer, r as integer, colour as integer) 'x, y coordinates of top left, width, height, radius of corner
  local integer dp
  dp=ssddraw(SSD.ffr, colour, x+r, y, w-2*r, h,  SSD_params()) 
  dp=ssddraw(SSD.fcircH,x+w-r-1, y+r, r, 1, h-2*r-1, colour, SSD_params())
  dp=ssddraw(SSD.fcircH,x+r, y+r, r, 2, h-2*r-1, colour, SSD_params())
end sub
 '
sub SSD.setrot(x%) 'sets screen rotation: 0=portrait, 1=portrait180, 2=landscape, 3-landscape180
    SSD_params(SSD_rot)=(x% and 3)
    local integer orient = 0
    select case x%
        case portrait
           orient = SSD1963_PAGE_ADDR_ORDER OR SSD1963_PAG_COL_ADDR_ORDER
        case portrait180
            orient = SSD1963_COL_ADDR_ORDER OR SSD1963_PAG_COL_ADDR_ORDER
        case landscape
        case landscape180
            orient = SSD1963_PAGE_ADDR_ORDER OR  SSD1963_COL_ADDR_ORDER
    end select
    SSDwritecommand(&H36) ' set addresss mode
    SSDwritedata(orient)
end sub
'
CFunction SSDdraw
	00000654
	'psend
	27bdffe0 afbf001c afb10018 afb00014 00a08821 8cb00030 00908026 aca40030 
	000427c3 32020001 10400006 aca40034 3c029d00 8c42001c 8ca40048 0040f809 
	24050007 32020002 10400007 32020004 3c029d00 8c42001c 8e240050 0040f809 
	24050007 32020004 10400007 32020008 3c029d00 8c42001c 8e240058 0040f809 
	24050007 32020008 10400007 32020010 3c029d00 8c42001c 8e240060 0040f809 
	24050007 32020010 10400007 32020020 3c029d00 8c42001c 8e240068 0040f809 
	24050007 32020020 10400007 32020040 3c029d00 8c42001c 8e240070 0040f809 
	24050007 32020040 10400007 32020080 3c029d00 8c42001c 8e240078 0040f809 
	24050007 32020080 50400007 32100100 3c029d00 8c42001c 8e240080 0040f809 
	24050007 32100100 12000007 3c109d00 3c029d00 8c42001c 8e240088 0040f809 
	24050007 3c109d00 8e02001c 8e240040 0040f809 24050005 8e02001c 8e240040 
	0040f809 24050006 8fbf001c 8fb10018 8fb00014 03e00008 27bd0020 

	'writeCommand
	27bdffe0 afbf001c afb10018 afb00014 00808821 00a08021 3c029d00 8c42001c 
	8ca40038 0040f809 24050005 02202021 02002821 0411FF93 00000000 8fbf001c 
	8fb10018 8fb00014 03e00008 27bd0020 

	'writeData
	27bdffe0 afbf001c afb10018 afb00014 00808821 00a08021 3c029d00 8c42001c 
	8ca40038 0040f809 24050006 02202021 02002821 0411FF7F 00000000 8fbf001c 
	8fb10018 8fb00014 03e00008 27bd0020 

	'setxy
	27bdffd0 afbf002c afb50028 afb40024 afb30020 afb2001c afb10018 afb00014 
	00809021 00a09821 00c0a021 00e0a821 8fb10040 3c109d00 8e02001c 8e240038 
	0040f809 24050005 2404002a 02202821 0411FF64 00000000 8e02001c 8e240038 
	0040f809 24050006 00122202 02202821 0411FF5C 00000000 8e02001c 8e240038 
	0040f809 24050006 02402021 02202821 0411FF54 00000000 8e02001c 8e240038 
	0040f809 24050006 00132202 02202821 0411FF4C 00000000 8e02001c 8e240038 
	0040f809 24050006 02602021 02202821 0411FF44 00000000 8e02001c 8e240038 
	0040f809 24050005 2404002b 02202821 0411FF3C 00000000 8e02001c 8e240038 
	0040f809 24050006 00142202 02202821 0411FF34 00000000 8e02001c 8e240038 
	0040f809 24050006 02802021 02202821 0411FF2C 00000000 8e02001c 8e240038 
	0040f809 24050006 00152202 02202821 0411FF24 00000000 8e02001c 8e240038 
	0040f809 24050006 02a02021 02202821 0411FF1C 00000000 8e02001c 8e240038 
	0040f809 24050005 2404002c 02202821 0411FF14 00000000 8fbf002c 8fb50028 
	8fb40024 8fb30020 8fb2001c 8fb10018 8fb00014 03e00008 27bd0030 

	'localfastfillrect
	27bdffc0 afbf003c afb70038 afb60034 afb50030 afb4002c afb30028 afb20024 
	afb10020 afb0001c 00808821 8fa20050 8fb00068 24d7ffff 8fa30058 02e3b821 
	2444ffff 8fa30060 00831821 8e070000 8e040010 1480000b 8e050008 00409821 
	0060a021 30a8ffff 0017a827 0115a821 32b5ffff 0006b027 0116b021 10000015 
	32d6ffff 24080001 1488000b 24080002 30e7ffff 00039827 00f39821 3273ffff 
	0002a027 00f43821 30f4ffff 00c0a821 10000018 02e0b021 14880007 24080003 
	00c09821 02e0a021 0040a821 10000011 0060b021 24080003 1488000e 30e7ffff 
	00179827 00f39821 3273ffff 0006a027 00f43821 30f4ffff 30a5ffff 0003a827 
	00b5a821 32b5ffff 0002b027 00b62821 30b6ffff 02e6b823 26f70001 00621023 
	24420001 72e2b802 24040044 02002821 0411FF1C 00000000 afb00010 02602021 
	02802821 02a03021 02c03821 0411FF3D 00000000 7e221340 001193c3 325201f8 
	00529025 7e332880 00118903 323101c0 02719825 3c029d00 8c42001c 8e040038 
	0040f809 24050006 12530005 26f1ffff 16e00025 2414ffff 1000002f 24020001 
	02402021 02002821 0411FE9F 00000000 02402021 02002821 0411FE9B 00000000 
	26f2ffff 12400023 26f2fffe 3c119d00 2413ffff 8e22001c 8e040040 0040f809 
	24050005 8e22001c 8e040040 0040f809 24050006 8e22001c 8e040040 0040f809 
	24050005 8e22001c 8e040040 0040f809 24050006 2652ffff 5653ffef 8e22001c 
	1000000d 24020001 02402021 02002821 0411FE7D 00000000 02602021 02002821 
	0411FE79 00000000 2631ffff 1634fff7 02402021 24020001 00001821 8fbf003c 
	8fb70038 8fb60034 8fb50030 8fb4002c 8fb30028 8fb20024 8fb10020 8fb0001c 
	03e00008 27bd0040 

	'DBitmap
	27bdff80 afbf007c afbe0078 afb70074 afb60070 afb5006c afb40068 afb30064 
	afb20060 afb1005c afb00058 afa40080 00a08021 afa60088 afa7008c 8fb20090 
	8fa70094 8fa60098 8fbe00a0 248affff 8fa40088 72442802 00aa5021 260bffff 
	8fa4008c 72442802 00ab5821 8fc40000 8fcc0010 1580000e 8fc50008 30a2ffff 
	000a1827 00431821 3063ffff 8fa90080 00094027 00481021 3042ffff afa40040 
	00a09821 01604021 10000012 02004821 240d0001 158d000d 240d0002 3082ffff 
	000b4827 00494821 3129ffff 00104027 00481021 3048ffff afa40040 00a09821 
	01401021 1000001d 8fa30080 518d0016 afa50040 240d0003 558d0019 7ce51340 
	3082ffff 000a4827 00494821 3129ffff 8fa30080 00034027 00481021 3048ffff 
	30a2ffff 000b1827 00431821 3063ffff 00105027 004a1021 3042ffff afa50040 
	10000006 00809821 00809821 01601021 02001821 01404021 8fa90080 7ce51340 
	000723c3 308401f8 00a42025 afa4002c 7ce42880 00073903 30e701c0 00873825 
	afa70030 7cc51340 000623c3 308401f8 00a42025 afa40034 7cc42880 00063143 
	30c601c0 00863025 afa60038 afbe0010 01202021 01002821 00603021 00403821 
	0411FE7E 00000000 3c029d00 8c42001c 8fc40038 0040f809 24050006 8fa4008c 
	18800072 8fbf007c afb20024 afb0004c 8fa50088 afa50050 00054823 afa90054 
	00850018 00001012 afa20048 afa00020 10000062 afa00044 0600001e 0213102b 
	5040001d 26310001 06a2001b 26310001 52e00019 26310001 8fa3001c 90620000 
	8fa40018 00441024 1040000b 8fa40034 8fa4002c 03c02821 0411FDD1 00000000 
	8fa40030 03c02821 0411FDCD 00000000 10000009 26310001 03c02821 0411FDC8 
	00000000 8fa40038 03c02821 0411FDC4 00000000 26310001 1632ffdf 26100001 
	26940001 8fa50024 8fa90088 12890017 02c5b021 8fa30020 02831021 24430007 
	28440000 0064100b 000210c3 8fa4009c 00821021 afa2001c 8fa50028 00b41823 
	000317c3 00021742 00621821 30630007 00621023 24090001 00491004 afa20018 
	02c08021 1000ffc4 00008821 8fa2003c 24420001 afa2003c 14520007 26b50001 
	1000000c 8fa90044 afa0003c 8fa30048 2463ffff afa30028 8fa40088 1880fff3 
	8fb60080 0000a021 8fa50040 1000ffd9 02a5b82b 25290001 afa90044 8fa2004c 
	8fa30024 00431021 afa2004c 8fa40020 8fa50050 00852021 afa40020 8fa90048 
	8fa20054 01224821 afa90048 8fa30044 8fa4008c 10640005 8fbf007c 1e40ffe2 
	8fb5004c 1000ffeb 8fa90044 8fbe0078 8fb70074 8fb60070 8fb5006c 8fb40068 
	8fb30064 8fb20060 8fb1005c 8fb00058 03e00008 27bd0080 

	'swap
	8c820000 8ca30000 ac830000 03e00008 aca20000 

	'drawline
	27bdffa0 afbf005c afbe0058 afb70054 afb60050 afb5004c afb40048 afb30044 
	afb20040 afb1003c afb00038 0080a821 00a08021 00c08821 afa7006c 8fa20070 
	14c20010 8fbe0074 000617c3 afa60010 afa20014 afa70018 00071fc3 afa3001c 
	afa60020 afa20024 afbe0028 00a03021 00053fc3 0411FE45 00000000 1000004b 
	8fbf005c 8fa2006c 54a20011 8fa4006c 00053fc3 afa60010 00068fc3 afb10014 
	afa50018 afa7001c 8fa30070 afa30020 000317c3 afa20024 afbe0028 00a03021 
	0411FE32 00000000 10000038 8fbf005c 00859823 001317c3 00539826 02629823 
	00a4202a 24020001 2403ffff 0064100a afa20030 8fa20070 0046a023 001417c3 
	0054a026 0054a023 8fa30070 00c3202a 24020001 2403ffff 0064100a afa20034 
	02939021 24160001 0000b821 afb10010 001117c3 afa20014 afb60018 afb7001c 
	afb60020 afb70024 afbe0028 02a02021 02003021 00103fc3 0411FE0C 00000000 
	00121040 0054182a 54600007 0262102a 8fa4006c 1204000c 02549021 8fa30030 
	02038021 0262102a 5440ffe9 afb10010 8fa40070 12240004 02539021 8fa20034 
	1000ffe2 02228821 8fbf005c 8fbe0058 8fb70054 8fb60050 8fb5004c 8fb40048 
	8fb30044 8fb20040 8fb1003c 8fb00038 03e00008 27bd0060 

	'drawchar
	27bdffc8 afbf0034 8fa20048 8fa9004c 8fa80050 8fab0054 7c042420 3c039d00 
	8c630054 00052880 00a31821 8c650000 90a30001 90ac0002 008c682a 15a00019 
	90aa0000 90ad0003 01ac6821 008d682a 51a00015 afa90010 afa80010 afa60014 
	afa70018 706a3002 000630c3 008c6023 70cc2002 24840004 00a42821 afa5001c 
	afab0020 00402021 01202821 01403021 00603821 0411FE69 00000000 10000013 
	8fbf0034 afa90010 00094fc3 afa90014 71485002 afaa0018 000a57c3 afaa001c 
	70681802 afa30020 00031fc3 afa30024 afab0028 00e02021 00403021 00023fc3 
	0411FDB4 00000000 8fbf0034 03e00008 27bd0038 

	'testchar
	8fa30010 8c680010 8c690000 70e61002 00442021 2502fffe 2c420002 10400009 
	8c630008 04800011 00001021 04a0000f 0124302a 14c0000d 0065302a 14c0000b 
	00000000 2d080002 11000008 24020001 04800006 00001021 04a00004 0064182a 
	14600002 0125282a 38a20001 03e00008 00000000 

	'pstring
	27bdffa8 afbf0054 afbe0050 afb7004c afb60048 afb50044 afb40040 afb3003c 
	afb20038 afb10034 afb00030 afa40020 8fa20068 afa2002c 8fb00070 8fb30088 
	8fb7008c 00c01821 0040a821 8fa50078 afa50024 8fa20080 afa20028 3c029d00 
	8c420054 00042080 00821021 8c420000 905e0000 92e20000 18400021 0200a021 
	73d08002 0200b021 00d08021 24110001 02169023 afb30010 02402021 02a02821 
	02803021 03c03821 0411FFB8 00000000 1040001c 02f11021 80440000 afb20010 
	afb50014 afb40018 afb3001c 8fa50020 8fa60024 8fa70028 0411FF6F 00000000 
	00404021 00604821 02001821 26310001 92e20000 0051102a 1040ffe5 02168021 
	ae630020 00031fc3 ae630024 8fa2002c ae620028 8fa3002c 000317c3 10000003 
	ae62002c 00004021 00004821 01001021 01201821 8fbf0054 8fbe0050 8fb7004c 
	8fb60048 8fb50044 8fb40040 8fb3003c 8fb20038 8fb10034 8fb00030 03e00008 
	27bd0058 

	'dcirch
	27bdff88 afbf0074 afbe0070 afb7006c afb60068 afb50064 afb40060 afb3005c 
	afb20058 afb10054 afb00050 00808021 00c0b021 8fb50088 8fb20090 8fa20098 
	8fb700a0 02408821 afa60038 afb5003c afa20034 30420010 1040003f 0080f021 
	00069fc3 02551021 afa20010 000217c3 afa20014 24020001 00001821 afa20018 
	afa3001c afa20020 afa30024 afb70028 02603821 0411FD14 00000000 02b21023 
	afa20010 000217c3 afa20014 24020001 00001821 afa20018 afa3001c afa20020 
	afa30024 afb70028 02002021 02c03021 02603821 0411FD04 00000000 00159fc3 
	02563821 afb50010 afb30014 24020001 00001821 afa20018 afa3001c afa20020 
	afa30024 afb70028 02002021 00e03021 00073fc3 0411FCF4 00000000 02d23823 
	afb50010 afb30014 24020001 00001821 afa20018 afa3001c afa20020 afa30024 
	afb70028 02002021 00e03021 00073fc3 0411FCE5 00000000 1a2000bb 24110001 
	02328823 00121023 00021040 afa20030 26b3ffff 26d4ffff 26b50001 26d60001 
	02408021 00009021 8fa30034 30630004 afa30040 8fa20034 30420002 afa20044 
	8fa30034 30630008 afa30048 8fa20034 30420001 afa20034 06200005 8fa30030 
	2610ffff 24630002 afa30030 02238821 8fa20040 10400023 26520001 8fa3003c 
	02031021 afa20010 000217c3 afa20014 24020001 00001821 afa20018 afa3001c 
	afa20020 afa30024 afb70028 03c02021 02c03021 00163fc3 0411FCB3 00000000 
	8fa30038 02033821 afb50010 001517c3 afa20014 24020001 00001821 afa20018 
	afa3001c afa20020 afa30024 afb70028 03c02021 00e03021 00073fc3 0411FCA2 
	00000000 8fa30044 10600024 8fa30048 8fa3003c 00701023 afa20010 000217c3 
	afa20014 24020001 00001821 afa20018 afa3001c afa20020 afa30024 afb70028 
	03c02021 02c03021 00163fc3 0411FC8E 00000000 8fa30038 02033821 afb30010 
	001317c3 afa20014 24020001 00001821 afa20018 afa3001c afa20020 afa30024 
	afb70028 03c02021 00e03021 00073fc3 0411FC7D 00000000 8fa30048 10600024 
	8fa30034 8fa20038 00503823 afb50010 001517c3 afa20014 24020001 00001821 
	afa20018 afa3001c afa20020 afa30024 afb70028 03c02021 00e03021 00073fc3 
	0411FC69 00000000 8fa3003c 02031021 afa20010 000217c3 afa20014 24020001 
	00001821 afa20018 afa3001c afa20020 afa30024 afb70028 03c02021 02803021 
	00143fc3 0411FC58 00000000 8fa30034 50600024 2673ffff 8fa20038 00503823 
	afb30010 001317c3 afa20014 24020001 00001821 afa20018 afa3001c afa20020 
	afa30024 afb70028 03c02021 00e03021 00073fc3 0411FC44 00000000 8fa3003c 
	00701023 afa20010 000217c3 afa20014 24020001 00001821 afa20018 afa3001c 
	afa20020 afa30024 afb70028 03c02021 02803021 00143fc3 0411FC33 00000000 
	2673ffff 2694ffff 26b50001 0250102a 10400005 26d60001 00121040 24420001 
	1000ff5d 02228821 8fbf0074 8fbe0070 8fb7006c 8fb60068 8fb50064 8fb40060 
	8fb3005c 8fb20058 8fb10054 8fb00050 03e00008 27bd0078 

	'fcirch
	27bdff90 afbf006c afbe0068 afb70064 afb60060 afb5005c afb40058 afb30054 
	afb20050 afb1004c afb00048 8fb10080 8fb40090 8fbe0098 afa40038 afa6003c 
	8fa20088 1a200076 02801821 24100001 02118023 0011b823 0017b840 2495ffff 
	24d3ffff 26940003 24960001 00009021 30440001 afa40030 24630001 afa30040 
	30420002 afa20034 06000004 8fa20030 2631ffff 26f70002 02178021 1040002a 
	26520001 8fa3003c 00711023 afa20010 000217c3 afa20014 24020001 00001821 
	afa20018 afa3001c 00111040 8fa30040 00621021 afa20020 000217c3 afa20024 
	8fa400a0 afa40028 03c02021 02c03021 00163fc3 0411FBDE 00000000 8fa20038 
	02223821 afb30010 001317c3 afa20014 24020001 00001821 afa20018 afa3001c 
	afb40020 001417c3 afa20024 8fa300a0 afa30028 03c02021 00e03021 00073fc3 
	0411FBCB 00000000 8fa40034 5080002b 26b5ffff 8fa3003c 00711023 afa20010 
	000217c3 afa20014 24020001 00001821 afa20018 afa3001c 00111040 8fa30040 
	00621021 afa20020 000217c3 afa20024 8fa400a0 afa40028 03c02021 02a03021 
	00153fc3 0411FBB2 00000000 8fa20038 00513823 afb30010 001317c3 afa20014 
	24020001 00001821 afa20018 afa3001c afb40020 001417c3 afa20024 8fa300a0 
	afa30028 03c02021 00e03021 00073fc3 0411FB9F 00000000 26b5ffff 2673ffff 
	26940002 0251102a 10400005 26d60001 00121040 24420001 1000ff9b 02028021 
	8fbf006c 8fbe0068 8fb70064 8fb60060 8fb5005c 8fb40058 8fb30054 8fb20050 
	8fb1004c 8fb00048 03e00008 27bd0070 

	'ftri
	27bdff90 afbf006c afb50068 afb40064 afb30060 afb2005c afb10058 afb00054 
	8fa20088 8fb100a0 8fb200a8 afa40038 afa6003c 8fa30080 afa30040 afa20044 
	8fa30090 afa30048 8fa30098 0046102a 10400009 afa3004c 27a4003c 27a50044 
	0411FD07 00000000 27a40038 27a50040 0411FD03 00000000 8fa2004c 8fa30044 
	0043102a 1040000a 8fa20044 27a4004c 27a50044 0411FCFA 00000000 27a40048 
	27a50040 0411FCF6 00000000 8fa20044 8fa3003c 0043102a 1040000a 8fb0003c 
	27a4003c 27a50044 0411FCED 00000000 27a40038 27a50040 0411FCE9 00000000 
	8fb0003c 8fa2004c 1602002d 8fb30044 8fa20038 afa20030 afa20034 8fa30040 
	0062202a 10800003 0043102a 10000003 afa30030 54400001 afa30034 8fa20048 
	8fa30030 0043182a 10600003 8fa30034 10000004 afa20030 0062182a 54600001 
	afa20034 8fa70030 8fa2003c afa20010 000217c3 afa20014 24020001 00001821 
	afa20018 afa3001c 8fa20034 24420001 00471023 afa20020 000217c3 afa20024 
	afb20028 02202021 00e03021 00073fc3 0411FB23 00000000 10000077 8fbf006c 
	00531026 0002102b 02629823 0270102a 14400036 24140001 0000a821 8fa20038 
	8fa3003c 02032823 8fa60040 00c23023 70a63002 8fa40044 00832023 00c4001a 
	008001f4 00002012 00822021 afa40030 8fa60048 00c23023 70a62802 8fa6004c 
	00c31823 00a3001a 006001f4 00001812 00621021 afa20034 0044102a 10400006 
	8fa70030 27a40030 27a50034 0411FC94 00000000 8fa70030 afb00010 001017c3 
	afa20014 8fa20034 24420001 00471023 afa20018 000217c3 afa2001c afb40020 
	afb50024 afb20028 02202021 00e03021 00073fc3 0411FAEA 00000000 26100001 
	0270102a 1040ffce 8fa20038 8fa2004c 0050182a 14600037 24140001 0000a821 
	8fa60040 8fa40038 8fa50048 8fa80044 02083823 00a61823 70e33802 00484023 
	00e8001a 010001f4 00001812 00661821 afa30030 8fa7003c 02073023 00a42823 
	70c53002 00471023 00c2001a 004001f4 00001012 00442021 afa40034 0083202a 
	10800006 8fa70030 27a40030 27a50034 0411FC5B 00000000 8fa70030 afb00010 
	001017c3 afa20014 8fa20034 24420001 00471023 afa20018 000217c3 afa2001c 
	afb40020 afb50024 afb20028 02202021 00e03021 00073fc3 0411FAB1 00000000 
	26100001 8fa2004c 0050182a 1060ffcd 8fa60040 8fbf006c 8fb50068 8fb40064 
	8fb30060 8fb2005c 8fb10058 8fb00054 03e00008 27bd0070 

	'main
	27bdffb8 afbf0044 afb00040 00808021 00e01021 8fa90058 8faa005c 8fa80060 
	8fab0064 8c830000 8c840004 00643825 14e00014 8fac0068 8ca40000 8cc70004 
	8cc60000 8c430004 8c420000 afa20010 afa30014 24020001 00001821 afa20018 
	afa3001c afa20020 afa30024 afa90028 0411FA85 00000000 00402021 100000cf 
	00602821 24070001 14670018 24070002 14800016 00000000 8ca40000 8cc70004 
	8cc60000 8c430004 8c420000 afa20010 afa30014 8d220000 8d230004 afa20018 
	afa3001c 8d420000 8d430004 afa20020 afa30024 afa80028 0411FA6B 00000000 
	00402021 100000b5 00602821 14670021 24070003 1480001f 00000000 8ca40000 
	8ca50004 8cc70004 8cc60000 8c430004 8c420000 afa20010 afa30014 8d220000 
	8d230004 afa20018 afa3001c 8d420000 8d430004 afa20020 afa30024 8d020000 
	8d030004 afa20028 afa3002c 8d620000 8d630004 afa20030 afa30034 afac0038 
	0411FEC1 00000000 8e040000 10000093 8e050004 1467000f 24070004 1480000d 
	00000000 8ca40000 8cc50000 8c460000 8d270000 8d420000 afa20010 afa80014 
	0411FBD6 00000000 8e040000 10000083 8e050004 1467000a 24070005 14800008 
	00000000 8ca40000 00c02821 0411F997 00000000 8e040000 10000078 8e050004 
	1467000a 24070007 14800008 00000000 8ca40000 00c02821 0411F9A0 00000000 
	8e040000 1000006d 8e050004 1467001e 24070008 1480001c 00000000 8ca40000 
	8ca50004 8cc70004 8cc60000 8c430004 8c420000 afa20010 afa30014 8d220000 
	8d230004 afa20018 afa3001c 8d420000 8d430004 afa20020 afa30024 8d020000 
	8d030004 afa20028 afa3002c afab0030 afac0034 0411FC71 00000000 00402021 
	1000004e 00602821 1467001d 24070009 1480001b 00000000 8ca40000 8ca50004 
	8cc70004 8cc60000 8c430004 8c420000 afa20010 afa30014 8d220000 8d230004 
	afa20018 afa3001c 8d420000 8d430004 afa20020 afa30024 8d020000 8d030004 
	afa20028 afa3002c afab0030 0411FDCA 00000000 8e040000 10000030 8e050004 
	14670019 2407000a 14800017 00000000 8ca40000 8ca50004 8cc70004 8cc60000 
	8c430004 8c420000 afa20010 afa30014 8d220000 8d230004 afa20018 afa3001c 
	8d420000 8d430004 afa20020 afa30024 afa80028 0411FC92 00000000 8e040000 
	10000016 8e050004 14670017 8fbf0044 14800016 8fb00040 8ca40000 8cc50000 
	8c460000 8d270000 8d420000 afa20010 8d020000 afa20014 8d620000 afa20018 
	afac001c 8fa2006c afa20020 0411FA58 00000000 00002021 00002821 00801021 
	00a01821 8fbf0044 8fb00040 03e00008 27bd0048 
End CFunction   'MIPS32 M4K
'
sub SSD_test
  local integer i, j, x0, y0, x1, y1, wi, ht, sz, bitmap(1)=(&HFFC1A191898583FF,0)
  local float tn
  SSD.setrot(landscape)
  wi= SSD_params(SSD_wi)
  ht= SSD_params(SSD_ht)
  SSD.fillscreen(rgb(black))
  SSD.drect(0, 0,wi,ht, rgb(red))
  for i=10 to 0 step -1
    SSD.pstring(1,wi/2-48, 70, str$(i,2), 2, rgb(green), rgb(black))
    pause 1000
  next i
  SSD.pstring(1,wi/2-48, 70, "  ", 2, rgb(green), rgb(black))
  for i=0 to 3
     SSD.setrot(i)
     SSD.Bitmap(10,10,8,8,3,rgb(blue),rgb(black),bitmap())
  next i  
  SSD.setrot(landscape)
  SSD.pstring(0,wi/2-112, 60, "Welcome to the", 2, rgb(green), rgb(black))
  SSD.pstring(0,wi/2-168, 100,  "Micromite MKII 4.7b8", 2, rgb(red), rgb(black))
  SSD.pstring(0,wi/2-108, 150, "in colour", 3, rgb(yellow), rgb(black))
  SSD.pstring(0,wi/2-176, 220,  "9-bit parallel SSD1963", 2, rgb(cyan), rgb(black))
  pause 4000
' demo fast line draw
  SSD.fillscreen(rgb(black))
  SSD.setrot(landscape180)
  x0=0:y0=0:y1=ht-1
  for i=0 to 44 step 2
    tn=tan(rad(i))
    x1=wi*tn
    SSD.dline(x0, y0, x1, y1, rgb(red))
  next i
  x1=wi-1
  for i=0 to 44 step 2
    tn=tan(rad(i))
    y1=ht*tn
    SSD.dline(x0, y0, x1, y1, rgb(red))
  next i
  SSD.drect(0, 0, wi, ht, rgb(blue))
  pause 2000

  SSD.fillscreen(rgb(black))
'demo fast text write
  SSD.setrot(landscape)

  for i=0 to ht-13 step 13
    SSD.pstring(0,0, i, "Hello world:Bye world:Hello world:Bye world:Hello world:Bye world:Hello world:Bye world", 1, rgb(white), rgb(black))
  next i
  pause 2000
  SSD.setrot(landscape180)
  SSD.fillscreen(rgb(black))
  for i=0 to ht-26 step 26
    SSD.pstring(0,0, i, "Micromite MK2:Invert:Micromite MK2:Invert", 2, rgb(red), rgb(black))
  next i
  pause 2000
  SSD.setrot(portrait180)
  SSD.fillscreen(rgb(black))
  for i=0 to wi-39 step 39
    SSD.pstring(0,0,i, "Rotate right:Rotate right:Rotate right", 3, rgb(yellow), rgb(black))
  next i
  pause 2000
  SSD.setrot(portrait)
  SSD.fillscreen(rgb(black))
  for i=0 to wi-52 step 52
    SSD.pstring(0,0,i, "Rotate Left:Rotate Left:Rotate Left", 4, rgb(blue), rgb(black))
  next i
  pause 2000
  SSD.fillscreen(rgb(black))
  ' test drawing rectangles
  for i=0 to 3
     SSD.setrot(i)
     SSD.frect(5, 5, 80, 90, rgb(blue))
     SSD.drect(2, 2, 86, 96, rgb(red))
  next i
  pause 2000
' test drawing rounded rectangles
  SSD.fillscreen(rgb(black))
  for i=0 to 3
     SSD.setrot(i)
     SSD.frndrect(5, 5, 80, 45, 10, rgb(blue))
     SSD.drndrect(2, 2, 86, 51, 12, rgb(red))
  next i
  pause 2000
  SSD.fillscreen(rgb(black))
' test drawing circles
  for i=0 to 3
     SSD.setrot(i)
     SSD.fcirc(40, 40, 35, rgb(blue))
     SSD.dcirc(40, 40, 39, rgb(red))
  next i
  pause 2000
' test drawing triangles and line drawing
  SSD.fillscreen(rgb(black))
  for i=0 to 3
     SSD.setrot(i)
     SSD.ftri(4, 4, 140, 6, 22, 124, rgb(blue))
     SSD.dtri(1, 2, 144, 4, 22, 127, rgb(red))
  next i
  pause 2000

end SUB
